@@ -22,12 +22,13 @@ See :ref:`Expressions` in the online documentation for more details.
 
 from __future__ import annotations
 
-from typing import Any, Optional, Type, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Optional, Type
 
 import pyarrow as pa
-from datafusion.common import DataTypeMap, NullTreatment, RexType
 from typing_extensions import deprecated
 
+from datafusion.common import DataTypeMap, NullTreatment, RexType
+
 from ._internal import expr as expr_internal
 from ._internal import functions as functions_internal
 
@@ -380,6 +381,22 @@ class Expr:
             value = pa.scalar(value)
         return Expr(expr_internal.Expr.literal(value))
 
+    @staticmethod
+    def string_literal(value: str) -> Expr:
+        """Creates a new expression representing a UTF8 literal value.
+
+        It is different from `literal` because it is pa.string() instead of
+        pa.string_view()
+
+        This is needed for cases where DataFusion is expecting a UTF8 instead of
+        UTF8View literal, like in:
+        https://github.com/apache/datafusion/blob/86740bfd3d9831d6b7c1d0e1bf4a21d91598a0ac/datafusion/functions/src/core/arrow_cast.rs#L179
+        """
+        if isinstance(value, str):
+            value = pa.scalar(value, type=pa.string())
+            return Expr(expr_internal.Expr.literal(value))
+        return Expr.literal(value)
+
     @staticmethod
     def column(value: str) -> Expr:
         """Creates a new expression representing a column."""
